# -*- coding: utf-8 -*-

import sqlite

class ExerciseSQL:
    """SQLite database for storing exercises"""
    def __init__(self, filename):
        # Is there exercise database if not create it
        try:
            open(filename, 'r+')
        except:
            self.create_database(filename)
        # Connect to the exercise database
        self.cx = sqlite.connect(filename)
        self.cu = self.cx.cursor()

    def create_database(self, filename):
        """Create empty database"""
        self.cx = sqlite.connect(filename)
        self.cu = self.cx.cursor()

        command = """
            create table exercise (
                id integer primary key,
                title varchar(128),
                type integer
                )"""

        self.cu.execute(command)
        self.cx.commit()

    def load_exercises(self, option='all'):
        """Load exercises from database"""
        self.cu.execute('select id, title, type from exercise')
        self.exercise_list = []
        # Get all data (title and type)
        if option == 'all':
            for row in self.cu:
                exercise = [row[0], unicode(row[1], 'utf-8'), row[2]]
                self.exercise_list.append(exercise)
        # Get only titles
        elif option == 'titles':
            for row in self.cu:
                exercise = unicode(row[1], 'utf-8')
                self.exercise_list.append(exercise)
        return self.exercise_list

    def insert_exercise(self, exercise_title, exercise_type):
        """Insert exercise into database"""
        self.cu.execute('insert into exercise (title, type) values (%s, %d)',
                        exercise_title.encode('utf-8'), exercise_type)
        self.cx.commit()

    def update_exercise(self, id, new_title, new_type):
        """Update exercise in the database"""
        self.cu.execute('update exercise set title=%s, type=%d where id=%s',
                        new_title.encode('utf-8'), new_type, id)
        self.cx.commit()

    def delete_exercise(self, id):
        """Delete exercise from database"""
        command = "delete from exercise where id=%s"
        self.cu.execute(command, id)
        self.cx.commit()
